/**
 * C++ Struct   Configuration
 *
 * Revision     1.0a (9th August 2010)
 * Programmer   Michael Allwright
 *
 * Purpose      Defines structure of static runtime values. used for global configuration of runtime
 *              parameters.
 *
 *              Also #defines a number of constants. Do not change these numbers! refer to CAIA Technical
 *              Report 080222B and the L3DGEComms library source code & documentation for more information
**/


#ifndef _LTCPMON_CONFIG_
#define _LTCPMON_CONFIG_

#define FLOW_PARAMS_MAX_STRLEN                          40

#define FLOW_PARAM_SLOWSTARTTHRESHOLD                   0    // SIFTR logfile column 8
#define FLOW_PARAM_CURRENTCONGESTIONWINDOW              1    // SIFTR logfile column 9
#define FLOW_PARAM_BANDWIDTHCONTROLLEDWINDOW            2    // SIFTR logfile column 10
#define FLOW_PARAM_CURRENTSENDINGWINDOW                 3    // SIFTR logfile column 11
#define FLOW_PARAM_CURRENTRECEIVEWINDOW                 4    // SIFTR logfile column 12
#define FLOW_PARAM_CURRENTSENDINGWINDOWSCALINGFACTOR    5    // SIFTR logfile column 13
#define FLOW_PARAM_CURRENTRECEIVINGWINDOWSCALINGFACTOR  6    // SIFTR logfile column 14
#define FLOW_PARAM_MAXSEGMENTSIZE                       7    // SIFTR logfile column 16
#define FLOW_PARAM_RTTESTIMATE                          8    // SIFTR logfile column 17
#define FLOW_PARAM_RETRANSMISSIONTIMEOUTLENGTH          9    // SIFTR logfile column 20
#define FLOW_PARAM_SOCKETSENDBUFFERSIZE                 10    // SIFTR logfile column 21
#define FLOW_PARAM_BYTESINSOCKETSENDBUFFER              11    // SIFTR logfile column 22
#define FLOW_PARAM_SOCKETRECEIVEBUFFERSIZE              12    // SIFTR logfile column 23
#define FLOW_PARAM_BYTESINSOCKETRECEIVEBUFFER           13    // SIFTR logfile column 24
#define FLOW_PARAM_INFLIGHTBYTES                        14    // SIFTR logfile column 25

#define STATMODES_PARAM_MAX_STRLEN                      15

#define STATMODE_PARAM_MEDIAN                           0
#define STATMODE_PARAM_STDDEV                           1
#define STATMODE_PARAM_MINIMUM                          2
#define STATMODE_PARAM_MAXIMUM                          3

#define VISUAL_METRIC_MAX_STRLEN                        20

#define VISUAL_METRIC_INFO                              0
#define VISUAL_METRIC_SPIN                              1
#define VISUAL_METRIC_BOUNCE_HEIGHT                     2
#define VISUAL_METRIC_SCALE                             3
#define VISUAL_METRIC_ROLL                              4
#define VISUAL_METRIC_COLOUR                            5
#define VISUAL_METRIC_BOUNCE_RATE                       6
#define VISUAL_METRIC_SOUND                             7

#include <string>
#include <vector>

struct Configuration
{
    static double visualStartTime;
    static double visualDuration;
    static double updatePeriod;
    static double playbackRatio;
    static std::string serverIP;
    static unsigned int serverPort;
    static std::string mappingsFilename;
    static std::string siftrLogFilename;
    static std::string entityFilename;
    static bool verboseMode;
    static bool durationLimitFlag;

    static const char flowCharacteristics[][FLOW_PARAMS_MAX_STRLEN];
    static const char flowCharacteristicsH[][FLOW_PARAMS_MAX_STRLEN + 10];
    static const char statisticsModes[][STATMODES_PARAM_MAX_STRLEN];
    static const char statisticsModesH[][STATMODES_PARAM_MAX_STRLEN + 15];
    static const char visualisationMetrics[][VISUAL_METRIC_MAX_STRLEN];
    static const char visualisationMetricsH[][VISUAL_METRIC_MAX_STRLEN + 5];

    static const unsigned int numStatisticsModes;
    static const unsigned int numFlowCharacteristics;
    static const unsigned int numVisualisationMetrics;
};

#endif
