/**
 * C++ Classes  Exceptions
 *
 * Revision     1.0a (9th August 2010)
 * Programmer   Michael Allwright
 *
 * Purpose      Defines are series of classes which inherit from the C++ std::runtime_error base class
 *              these classes are exceptions which generally define behaviour of the application under
 *              failure conditions in terms of printing error messages, printing usage information,
 *              and passing back whether the application should close under these conditions
 *
 * TODO         Destroy this class and remove every trace of it from history. Replace with a single,
 *              more configurable class which should forfil the same purposes.
 *              Use bitflags (1,2,4.. %), switches, #defines to control the flow and behaviour.
 *
 * NOTE         This class is ugly, but it works, fix other stuff first!
**/

#ifndef _LTCPMON_EXCEPTIONS_
#define _LTCPMON_EXCEPTIONS_

#include <stdexcept>
#include <string>

/* Base exception case deriving from runtime_error */
class siftrDaemonRuntimeException : public std::runtime_error
{
   public:
    siftrDaemonRuntimeException(const std::string& message, bool exitRequired, bool printUsage)
      : std::runtime_error(message), exitRequiredFlag(exitRequired), printUsageFlag(printUsage)  {};

    bool exitRequiredFlagSet() { return exitRequiredFlag; }
    bool printUsageFlagSet() { return printUsageFlag; }

    private:
    bool exitRequiredFlag;
    bool printUsageFlag;
};


/* For errors occuring during command line parsing */
class badSyntaxException : public siftrDaemonRuntimeException
{
    public:
    badSyntaxException(const std::string& message)
      : siftrDaemonRuntimeException(message,true,true) {};
};

/* For errors parsing the logfile during intialisation */
class badLogFileException : public siftrDaemonRuntimeException
{
    public:
    badLogFileException(const std::string& message)
      : siftrDaemonRuntimeException(message,true,false) {};
};

class fileAccessDeniedException : public siftrDaemonRuntimeException
{
    public:
    fileAccessDeniedException(const std::string& message)
      : siftrDaemonRuntimeException(message,true,false) {};
};


class fileNotFoundException : public siftrDaemonRuntimeException
{
    public:
    fileNotFoundException(const std::string& message)
      : siftrDaemonRuntimeException(message,true,false) {};
};

/* For data parsing is complete exception object */
class parsingException : public siftrDaemonRuntimeException
{
    public:
    parsingException(const std::string& message)
      : siftrDaemonRuntimeException(message,false,false) {};
};

/* For data parsing is complete exception object */
class noMoreDataException : public siftrDaemonRuntimeException
{
    public:
    noMoreDataException(const std::string& message)
      : siftrDaemonRuntimeException(message,false,false) {};
};



#endif
